*Program: Trivariate normal (generalised Heckman model) for Mastitis data,
*uses NAG(1996) routines to optimize the likelihood  
	  implicit none
	  character tail
	  integer ni
*ni is number of cows
	  parameter (ni=107)
        integer nout,nin,nin1
        parameter (nin=5,nin1=7,nout=6)
	  integer nmax,nclmax,ncnmax
	  parameter  (nmax=30,nclmax=30,ncnmax=30)
        integer     lda, ldcj, ldr
        parameter   (lda=nclmax, ldcj=ncnmax, ldr=nmax)
        integer     liwork, lwork
        parameter   (liwork=500, lwork=3000)
        integer i,j,ifail,iter,n,nclin,ncnln
     +  ,istate(nmax+nclmax+ncnmax), iuser(1),iwork(liwork),k,
     +  rd(ni),ex(ni,5)
        real*8 a(lda,nmax), bl(nmax+nclmax+ncnmax),
     +  c(ncnmax),bu(nmax+nclmax+ncnmax), cjac(ldcj,nmax)
     +  ,objf,clamda(nmax+nclmax+ncnmax),objgrd(nmax), r(ldr,nmax),
     +  user(1),work(lwork),x(nmax),confun
	  real*8 y(ni,2),g01eaf,q6,q8,q7,q,muy1,muy2,vary1,vary2
        external e04ucf,objfun,e04uef,g01eaf
	  common y,rd,ex
        data tail/'l'/
	  open(nin,file='likp1.dat',status='old')
	  open(nin1,file='likp2.dat',status='old')
        open(nout,file='likpp.out',status='unknown')
*Reading number of parameters (n), Lower and upper bounds for the 
*parameters (bl and bu) and initial values (x) from file:likp1.dat	
	  read(nin,*) n,nclin,ncnln
        read (nin,*) (bl(i), i=1,n+nclin+ncnln)
        read (nin,*) (bu(i),i=1,n+nclin+ncnln)
        read (nin,*) (x(i),i=1,n)
*reading data (rd; for missing indicator, y; for two observations 
*and ex; for selected year) from file likp2.dat
	  do 10 i=1,107
	  read (nin1,*)rd(i),y(i,1),y(i,2),(ex(i,j),j=1,5)
  10    continue	
	  call e04uef('derivative level=0')
        call e04uef('major iteration limit=200')
        ifail=-1
*Calling the routine for optimizating the likelihood (objfun)
*Sending output to file likpp.out (only the estimates)
        call e04ucf(n,nclin,ncnln,lda,ldcj,ldr,a,bl,bu,confun,objfun,
     +  iter,istate,c,cjac,clamda,objf,objgrd,r,x,iwork,
     +  liwork,work,lwork,iuser,user,ifail)
	  stop
        end

*Subroutine for the minus log-likelihood as object function 
        subroutine objfun(mode,n,x,objf,objgrd,nstate,iuser,user)
        implicit none
        character tail
        integer ni
        parameter (ni=107)
        integer mode,n,nstate,ifail
        integer i,iuser(*),rd(ni),ex(ni,5)
        real*8 objgrd(n),user(*),g01eaf,objf
        real*8 y(ni,2)
        real*8 f1,f2,q,q1,q2,q3,q4,q5,q6,q7,q8,q9,sum,x(n)
        external g01eaf
        common y,rd,ex
        data tail/'l'/
        if(mode.eq.0 .or. mode.eq.2)then
        objf=0.0
*Using a loop to obtain the summation of the -loglikelihood
        do 120 i=1,ni
	  sum=0
*Likelihood for individuals with both responses 
        if(rd(i) .eq. 1)then
	  q1=x(1)+x(2)*ex(i,1)+x(3)*ex(i,2)+x(4)*ex(i,3)+x(5)*ex(i,4)
	  f1=dexp((-1.0/(2*x(6)**2))*(y(i,1)-q1)**2)/(2.506628*x(6))
	  q2=x(1)+x(2)*ex(i,1)+x(3)*ex(i,2)+x(4)*ex(i,3)+x(5)*ex(i,4)+
     +  x(7)+(x(9)*x(8)/x(6))*(y(i,1)-q1)
	  q3=sqrt(x(8)**2-(x(9)**2*x(8)**2))
	  f2=dexp((-1/(2*q3**2))*(y(i,2)-q2)**2)/(2.506628*q3)
	  q4=(x(6)*x(8))**2*(1-x(9)**2)
	  q5=x(6)*x(10)*x(8)**2-x(6)*x(8)**2*x(9)*x(11)
	  q6=x(6)**2*x(8)*x(11)-x(6)**2*x(8)*x(10)*x(9)
	  q7=x(12)+x(13)*ex(i,4)+x(14)*ex(i,1)+x(15)*ex(i,2)+x(16)*ex(i,3)+q5
     +  *(y(i,1)-q1)/q4
     +  +q6*(y(i,2)-x(1)-x(2)*ex(i,1)-x(3)*ex(i,2)-x(4)*ex(i,3)-
     +  x(5)*ex(i,4)-x(7))/q4
	  q8=(x(10)**2-2*x(9)*x(10)*x(11)+x(11)**2)/(1-x(9)**2)
	  if( q8 .ge. 1 ) then
	  q8=0
	  q7=-3
        end if
	  q8=(1.0-q8)**0.5
	  q9=q7/q8
        ifail=0
        q=g01eaf(tail,q9,ifail)
        sum=f1*f2*q
*Likelihood for individuals with only the first response, i.e. they
*dropout at t=2.
        else
        q1=x(1)+x(2)*ex(i,1)+x(3)*ex(i,2)+x(4)*ex(i,3)+x(5)*ex(i,4)
	  f1=dexp((-1/(2*x(6)**2))*(y(i,1)-q1)**2)/(2.506628*x(6))
	  q2=x(12)+x(13)*ex(i,4)+x(14)*ex(i,1)+x(15)*ex(i,2)+x(16)*ex(i,3)
     +  +x(10)*(y(i,1)-q1)/x(6)
	  q3=sqrt(1.0-x(10)**2)
	  q6=-q2/q3
        ifail=0
        q=g01eaf(tail,q6,ifail)
        sum=f1*q
        endif
        if(sum.eq.0.0)sum=1.0d-300
        objf=(-1)*dlog(sum)+objf
 120    continue
        endif
        return
        end
